#include <Tools.h>
#include "ExternalSort.h"

const std::string VERSION = "0.1b";
const std::string DATE = "6 February 2004";

Tools::IndexOutOfBoundsException::IndexOutOfBoundsException(int i)
{
	std::ostringstream s;
	s << "Invalid index " << i;
	m_error = s.str();
}

std::string Tools::IndexOutOfBoundsException::what()
{
	return "IndexOutOfBoundsException: " + m_error;
}

Tools::IllegalArgumentException::IllegalArgumentException(std::string s) : m_error(s)
{
}

std::string Tools::IllegalArgumentException::what()
{
	return "IllegalArgumentException: " + m_error;
}

Tools::IllegalStateException::IllegalStateException(std::string s) : m_error(s)
{
}

std::string Tools::IllegalStateException::what()
{
	return "IllegalStateException: " + m_error + "\nPlease contact " + EMAIL;
}

Tools::EndOfStreamException::EndOfStreamException(std::string s) : m_error(s)
{
}

std::string Tools::EndOfStreamException::what()
{
	return "EndOfStreamException: " + m_error;
}

Tools::ResourceLockedException::ResourceLockedException(std::string s) : m_error(s)
{
}

std::string Tools::ResourceLockedException::what()
{
	return "ResourceLockedException: " + m_error;
}

Tools::InvalidPageException::InvalidPageException(long id)
{
	std::ostringstream s;
	s << "Unknown page id " << id;
	m_error = s.str();
}

std::string Tools::InvalidPageException::what()
{
	return "InvalidPageException: " + m_error;
}

Tools::NotSupportedException::NotSupportedException(std::string s) : m_error(s)
{
}

std::string Tools::NotSupportedException::what()
{
	return "NotSupportedException: " + m_error;
}

Tools::Variant::Variant() : varType(VT_EMPTY)
{
}

Tools::Variant Tools::PropertySet::getProperty(std::string property)
{
	std::map<std::string, Variant>::iterator it = m_propertySet.find(property);
	if (it != m_propertySet.end()) return (*it).second;
	else return Variant();
}

void Tools::PropertySet::setProperty(std::string property, Variant& v)
{
	m_propertySet.insert(std::pair<std::string, Variant>(property, v));
}

void Tools::PropertySet::removeProperty(std::string property)
{
	std::map<std::string, Variant>::iterator it = m_propertySet.find(property);
	if (it != m_propertySet.end()) m_propertySet.erase(it);
}

Tools::IObjectStream* Tools::externalSort(IObjectStream& source, unsigned long bufferSize)
{
	return new ExternalSort(source, bufferSize);

}

Tools::IObjectStream* Tools::externalSort(IObjectStream& source, IObjectComparator& comp, unsigned long bufferSize)
{
	return new ExternalSort(source, comp, bufferSize);
}

Tools::Interval::Interval() : m_type(IT_CLOSED), m_low(0.0), m_high(0.0)
{
}

Tools::Interval::Interval(IntervalType t, double l, double h) : m_type(t), m_low(l), m_high(h)
{
	assert(l < h);
}

double Tools::Interval::getLowerBound() const
{
	return m_low;
}

double Tools::Interval::getUpperBound() const
{
	return m_high;
}

bool Tools::Interval::intersectsInterval(const IInterval& i) const
{
	return intersectsInterval(i.getIntervalType(), i.getLowerBound(), i.getUpperBound());
}

bool Tools::Interval::intersectsInterval(IntervalType type, const double low, const double high) const
{
	if (m_low > high || m_high < low) return false;
	if ((m_low > low && m_low < high) || (m_high > low && m_high < high)) return true;

	switch (m_type)
	{
	case IT_CLOSED:
		if (m_low == high)
		{
			if (type == IT_CLOSED || type == IT_LEFTOPEN) return true;
			else return false;
		}
		else if (m_high == low)
		{
			if (type == IT_CLOSED || type == IT_RIGHTOPEN) return true;
			else return false;
		}
		break;
	case IT_OPEN:
		if (m_low == high || m_high == low) return false;
		break;
	case IT_RIGHTOPEN:
		if (m_low == high)
		{
			if (type == IT_CLOSED || type == IT_LEFTOPEN) return true;
			else return false;
		}
		else if (m_high == low)
		{
			return false;
		}
		break;
	case IT_LEFTOPEN:
		if (m_low == high)
		{
			return false;
		}
		else if (m_high == low)
		{
			if (type == IT_CLOSED || type == IT_RIGHTOPEN) return true;
			else return false;
		}
		break;
	}

	return true;
}

bool Tools::Interval::containsInterval(const IInterval& i) const
{
	double low = i.getLowerBound();
	double high = i.getUpperBound();
	IntervalType type = i.getIntervalType();

	if (m_low < low && m_high > high) return true;
	if (m_low > low || m_high < high) return false;

	switch (m_type)
	{
	case IT_CLOSED:
		break;
	case IT_OPEN:
		if ((m_low == low && m_high == high && type != IT_OPEN) ||
			(m_low == low && (type == IT_CLOSED || type == IT_RIGHTOPEN)) ||
			(m_high == high && ( type == IT_CLOSED || type == IT_LEFTOPEN))) return false;
		break;
	case IT_RIGHTOPEN:
		if (m_high == high && (type == IT_CLOSED || type == IT_LEFTOPEN))return false;
		break;
	case IT_LEFTOPEN:
		if (m_low == low && (type == IT_CLOSED || type == IT_RIGHTOPEN))return false;
		break;
	}

	return true;
}

Tools::IntervalType Tools::Interval::getIntervalType() const
{
	return m_type;
}

void Tools::Random::initialize()
{
	static bool bCalled = false;

	if (bCalled == false)
	{
		srand(time(NULL));
		bCalled = true;
	}
}

long Tools::Random::nextUniformLong(long low, long high)
{
	initialize();
	return (long) floor(((double) low + ((double) (high - low)) * rand()));
}

int Tools::Random::nextUniformInt(int low, int high)
{
	initialize();
	return (int) floor(((double) low + ((double) (high - low)) * rand()));
}

double Tools::Random::nextUniformDouble(double low, double high)
{
	initialize();
	return (low + ((high - low) * rand()));
}

double Tools::Random::nextUniformDouble()
{
	initialize();
	return rand();
}

// mean 0.0, standard deviation 1.0
double Tools::Random::nextGaussianDouble()
{
	initialize();
	static bool haveNextNextGaussian = false;
	static double nextNextGaussian;

	if (haveNextNextGaussian)
	{
		haveNextNextGaussian = false;
		return nextNextGaussian;
	}
	else
	{
		double v1, v2, s;

		do
		{
			v1 = 2 * rand() - 1;   // between -1.0 and 1.0
			v2 = 2 * rand() - 1;   // between -1.0 and 1.0
			s = v1 * v1 + v2 * v2;
		}
		while (s >= 1 || s == 0);

		double multiplier = std::sqrt(-2 * std::log(s)/s);
		nextNextGaussian = v2 * multiplier;
		haveNextNextGaussian = true;
		return v1 * multiplier;
	}
}

double Tools::Random::nextGaussianDouble(double m, double std)
{
	initialize();
 	return m + (std * nextGaussianDouble());
}

double Tools::Random::nextZipfDouble(double low, double high, double p)
{
	double x;
	int i;
	double r, HsubV, sum;
	int V = 100;

	/* calculate the V-th harmonic number HsubV. WARNING: V>1 */
	HsubV = 0.0;
	for(i = 1; i <= V; i++) HsubV += 1.0 / pow((double) i, p);

	r = nextUniformDouble() * HsubV;
	sum = 1.0; i=1;
	while(sum < r)
	{
		i++;
		sum += 1.0/ pow((double)i, p);
	}

	/* i follows Zipf distribution and lies between 1 and V */
	/* x lies between 0. and 1. and then between low and high */
	x = ((double) i - 1.0) / ((double) V - 1.0);
	x = (high - low) * x + low;

	return x;
}

bool Tools::Random::flipCoin()
{
	if (nextUniformDouble() >= 0.5) return true;
	return false;
}

Tools::ResourceUsage::ResourceUsage()
{
//	reset();
}

void Tools::ResourceUsage::start()
{
//	struct timezone dummy;
//	if (getrusage(RUSAGE_SELF, &m_tmpRU) != 0) throw IllegalStateException("Tools::ResourceUsage::start: getrusage failed.");;
//  	if (gettimeofday(&m_tmpTV, &dummy) != 0) throw IllegalStateException("Tools::ResourceUsage::start: gettimeofday failed.");;
}

void Tools::ResourceUsage::stop()
{
//	struct timezone dummy;
//  	struct timeval dif;
//	struct rusage ru;
//  	struct timeval tv;
//	if (getrusage(RUSAGE_SELF, &ru) != 0) throw IllegalStateException("Tools::ResourceUsage::stop: getrusage failed.");
//	if (gettimeofday(&tv, &dummy) != 0) throw IllegalStateException("Tools::ResourceUsage::stop: gettimeofday failed.");
//	// total_time
//	subtractTimeval(dif, tv, m_tmpTV);
//	addTimeval(m_totalTime, dif);
//	// system_time
//	subtractTimeval(dif, ru.ru_stime, m_tmpRU.ru_stime);
//	addTimeval(m_systemTime, dif);
//	// user_time
//	subtractTimeval(dif, ru.ru_utime, m_tmpRU.ru_utime);
//  	addTimeval(m_userTime, dif);
//	// readIO, writeIOs
//	m_readIO += ru.ru_inblock - m_tmpRU.ru_inblock;
//	m_writeIO += ru.ru_oublock - m_tmpRU.ru_oublock;
//	// maximum resident set size
//	m_peakMemory = std::max(m_peakMemory, ru.ru_maxrss);
//	// page faults
//	m_pageFaults += ru.ru_majflt - m_tmpRU.ru_majflt;
}

void Tools::ResourceUsage::reset()
{
	//m_pageFaults = 0;
	//m_readIO = 0;
	//m_writeIO = 0;
	//m_peakMemory = 0;
	//m_totalTime.tv_sec = 0;
	//m_totalTime.tv_usec = 0;
	//m_userTime.tv_sec = 0;
	//m_userTime.tv_usec = 0;
	//m_systemTime.tv_sec = 0;
	//m_systemTime.tv_usec = 0;
}

double Tools::ResourceUsage::combineTime(const struct timeval& t)
{
	//return static_cast<double>(t.tv_sec) + static_cast<double>(t.tv_usec) / 1000000.0;
	return(0);
}

void Tools::ResourceUsage::addTimeval(struct timeval& result, const struct timeval& a)
{
	/*result.tv_sec += a.tv_sec;
	result.tv_usec += a.tv_usec;

	if (result.tv_usec > 1000000)
	{
		long div = result.tv_usec / 1000000;
		result.tv_sec += div;
		result.tv_usec -= div * 1000000;
	}*/
}

void Tools::ResourceUsage::subtractTimeval(struct timeval& result, const struct timeval& a, const struct timeval& b)
{
//	result.tv_sec = a.tv_sec - b.tv_sec;
//	result.tv_usec = a.tv_usec - b.tv_usec;
//	if (result.tv_usec < 0)
//	{
//		result.tv_sec -= 1;
//    	result.tv_usec += 1000000;
//    }
}

double Tools::ResourceUsage::getTotalTime()
{
/*	return combineTime(m_totalTime);*/
	return(0);
}

double Tools::ResourceUsage::getUserTime()
{
	/*return combineTime(m_userTime);*/
	return(0);
}

double Tools::ResourceUsage::getSystemTime()
{
	//return combineTime(m_systemTime);
	return(0);
}

long Tools::ResourceUsage::getPageFaults()
{
	//return m_pageFaults;
	return(0);
}

long Tools::ResourceUsage::getReadIO()
{
	//return m_readIO;
	return(0);
}

long Tools::ResourceUsage::getWriteIO()
{
	//return m_writeIO;
	return(0);
}

long Tools::ResourceUsage::getPeakMemoryUsage()
{
	//return m_peakMemory;
	return(0);
}
