/** @if INTERNAL
 *
 *  @file       src/globals.cpp
 *  @author     Taka
 *  @date       23nov97
 *  
 *  @copyright  (c) T. Muraoka 1997 (all rights reserved).
 *              Unauthorized use of this code is prohibited.
 *  
 *  @brief      Global definitions.
 * 
 *  @licence    This software is provided 'as-is', without any express 
 *              or implied warranty.
 *
 *              In no event will the author be held liable for any damages 
 *              arising from the use of this software.
 *  
 *              Permission is granted to anyone to use this software 
 *              for any purpose, including commercial applications, 
 *              and to alter it and redistribute it freely, subject to 
 *              the following restrictions: 
 *
 *              - The origin of this software must not be misrepresented; 
 *                you must not claim that you wrote the original software. 
 *                If you use this software in a product, an acknowledgment 
 *                in the product documentation is requested but not required. 
 *
 *              - Altered source versions must be plainly marked as such, 
 *                and must not be misrepresented as being the original software. 
 *                Altered source is encouraged to be submitted back to 
 *                the original author so it can be shared with the community. 
 *                Please share your changes. 
 *
 *              - This notice may not be removed or altered from any 
 *                source distribution. 
 *  
 *  @endif
 */

#include <iostream>

//#include "lib.hpp"
#include "log.h"          

using namespace std ;

namespace debug
{
namespace log
{

/* --- LOCAL DATA ----------------------------------------------------- */

static CMessageLog*     gpMsgLog = NULL ;

/* -------------------------------------------------------------------- */

static CMessageLog&
theMessageLog( CMessageGroup* pMsgGroup , const char* pFileName , size_t lineNo )
{
    // return the global CMessageLog
    if ( gpMsgLog != NULL )
        return (*gpMsgLog)( pMsgGroup , pFileName , lineNo ) ;
    else 
    {
        static CMessageLog msgLog( clog ) ;
        return msgLog( pMsgGroup , pFileName , lineNo ) ;
    }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

CMessageLog&    theMessageLog() { return theMessageLog(NULL,NULL,0) ; }
CMessageLog&    theMessageLog( const char* p , size_t n ) { return theMessageLog(NULL,p,n) ; }
CMessageLog&    theMessageLog( CMessageGroup& mg , const char* p , size_t n ) { return theMessageLog(&mg,p,n) ; }

/* -------------------------------------------------------------------- */

CMessageLog*
setMessageLog( CMessageLog* pMsgLog )
{
    CMessageLog* pPrevMsgLog ;

    // set the new message log
    pPrevMsgLog = gpMsgLog ;
    gpMsgLog = pMsgLog ;

    return pPrevMsgLog ;
}

} // End namespace log
} // End namespace debug
