/** @if INTERNAL
 *
 *  @file       src/streambf.cpp
 *  @author     Taka
 *  @date       23oct97
 *  
 *  @copyright  (c) T. Muraoka 1997 (all rights reserved).
 *              Unauthorized use of this code is prohibited.
 *  
 *  @brief      Implements the CMessageLogStreamBuf class.
 * 
 *  @licence    This software is provided 'as-is', without any express 
 *              or implied warranty.
 *
 *              In no event will the author be held liable for any damages 
 *              arising from the use of this software.
 *  
 *              Permission is granted to anyone to use this software 
 *              for any purpose, including commercial applications, 
 *              and to alter it and redistribute it freely, subject to 
 *              the following restrictions: 
 *
 *              - The origin of this software must not be misrepresented; 
 *                you must not claim that you wrote the original software. 
 *                If you use this software in a product, an acknowledgment 
 *                in the product documentation is requested but not required. 
 *
 *              - Altered source versions must be plainly marked as such, 
 *                and must not be misrepresented as being the original software. 
 *                Altered source is encouraged to be submitted back to 
 *                the original author so it can be shared with the community. 
 *                Please share your changes. 
 *
 *              - This notice may not be removed or altered from any 
 *                source distribution. 
 *  
 *  @endif
 */

#include <iostream>
#include <cassert>

#include "log.h"          

namespace debug
{
namespace log
{

/* --- CONSTRUCTOR ---------------------------------------------------- */

/// @constructors
//@{

CMessageLogStreamBuf::CMessageLogStreamBuf( CMessageLog& msgLog )
    : mMsgLog( msgLog )
{
}

//@}

/* --- DESTRUCTOR ----------------------------------------------------- */

/// @destructor
//@{

CMessageLogStreamBuf::~CMessageLogStreamBuf()
{
}

//@}

/* -------------------------------------------------------------------- */

int
CMessageLogStreamBuf::overflow( int ch )
{
    // pass the character on to the parent CMessageLog object for processing
    return mMsgLog.onStreamBufChar( ch )  ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

int
CMessageLogStreamBuf::underflow()
{
    // NOTE: This streambuf is write-only - we should never get here!
    assert( false /*"Should never get here!"*/ ) ;

    return EOF ;
}

} // End namespace log
} // End namespace debug
