#ifndef _PREPARED_STATEMENT_H
#define _PREPARED_STATEMENT_H

#include "odbc.h"
#include<string>
#include<vector>

//Forward References
class otl_connect;
class ResultSet;

/**
 *
 * first version
 *
 * \date 02-20-2004
 *
 * \author
 *
 * This class uses the otl stream interface to give an interface as simliar
 * to Java's resultset. 
 *
 * \bug 
 *
 */
class PreparedStatement
{
	//friend classes
	friend 
	class Connection;

	private:
	
	/**
	* Constructs a prepared statement given the name
	* \param sqlStatement 
	* \param otlConnectPtr 
	* \return 
	*/
	PreparedStatement(std::string sqlStatement,
					  otl_connect* otlConnectPtr,
					  const unsigned int bufferSize);
	
public:
	/**
	* Sets the string at the index
	* \param str 
	* \param index 
	* \return 
	*/
	void setString(std::string str,int index);

	/**
	* Executes the prepared statement which must be an sql update
	* int the form of INSERT, DELETE and UPDATE
	*@pre the set on each and every column should be called
	*/
	void executeUpdate();

	/**
	* Executes the prepared statement which must be an sql update
	* int the form of INSERT, DELETE and UPDATE
	*@pre the set on each and every column should be called
	*/
	ResultSet* executeQuery();
	
	/**
	* Sets the float value at the column index
	* \param item 
	* \param index 
	* \return 
	*/
	void setFloat(float item, int index);

	/**
	* Set the integer value at the column index
	* \param item 
	* \param index 
	* \return 
	*/
	void setInt(int item,int index);


	/**
	* Sets the double value at the column index
	* \param item 
	* \param index 
	* \return 
	*/
	void setDouble(double item,int index);	

	/**
	* This methods flushes the buffer to the data base.
	*/
	void PreparedStatement :: flush();

	/**
	* Destroys the prepared statment deallocates the array.
	* \return 
	*/
	PreparedStatement::~PreparedStatement();
	//Private member methods of the class
	private:
		/**
		* This method uses the arrayVaraibles and the vec type to 
		* bind the actual values in place of place holders in the query.
		* and executes the sql_statement
		*/
		void execute();

		/**
		* This method is used to recover from an error in the 
		* constructor
		*/
		void recover(int i);
//Private members variables in the class
	private :
	//Stores the variables to be bound
	void **arrayVariables; 
	//Stores the length of the array
	int length;
	//Stores the sqlstatement
	std::string sqlstatement;
	//Stores a pointer to the input stream
	otl_stream *stream;
	//Stores the type information
	std::vector<int> vecType;

};




#endif //_PREPARED_STATEMENT_H