#ifndef _BRUSH_ENTRIES_H
#define _BRUSH_ENTRIES_H
#include <cache/Brush.h>
#include <list>
#include <cache/ICacheEntry.h>

namespace cache
{
	/*
	 *	A list of cache entries that belong to a brush
	 * Constructed when the brush entries are to be replaced
	 * Maintains the list in the sorted order and returns a reverse iterator 
	 * when required.
	 */
	class BrushEntries
	{

	public:
		//Creates a sorted list of brush entries
		BrushEntries::BrushEntries(std::list<ICacheEntry*>& listCacheEntry,
								   Brush *currentBrush);

		//Construts a new cache entry
		BrushEntries(std::list<ICacheEntry*>& listCacheEntry,
					 Brush* currentBrush,
					 bool front);

		//Remove the contents of the brushEntries
		~BrushEntries();

		Brush& getBrush();
		/**
		* This creates a new brush and returns a pointer to the brush
		* \param x 
		* \param y 
		* \param lod 
		* \return 
		*/
		Brush* getLeftOverBrush();
		
		/*
		 *	Returns the left end to the brush entry
		 * @return CacheEntry<bool>*
		 */
		ICacheEntry* getLeftEnd();
		
		/*
		 *	Returns the right end to the brush entry
		 * @return CacheEntry<bool>*
		 */
		ICacheEntry* getRightEnd();

	
		/*
		 * Returns the cache entry that would have been deleted	
		 */
		ICacheEntry* peek();

		/*
		* @ return usigned long
		* Returns the size of the list	
		*/
		unsigned long size();

		/*
		* Fetches the CacheEntry, Note the direction has to be initialized
		* before using this method.
		* @pre setDirection
		*	@return ICacheEntry*
		*/
		ICacheEntry* getNext();

		/*
		*	Erases the brush entry 
		*  last fetched using getNext
		*/
		void erase();

		/*
		*	Set leftToRight = true for lefttoright else set to false
		*/
		void setDirection(bool leftToRight);
		
		/*
		 *	Used only for debugging purposes.
		 *  Returns ids of all the nodes in the brush entries
		 */
		int* getIDs(unsigned int& length);
		

	private:
		//if front = true
		//the entries are remove from the front 
		//else removed from the back of the vector.
		bool front;

		//Stores the brush for which it holds cache entries
		Brush brush;

		//Stores the list of the cache entries
		//that belong to the brush in a sorted order.
		std::deque<ICacheEntry*>* dequeCacheEntryPtr;

		std::deque<ICacheEntry*>::iterator it;
		std::deque<ICacheEntry*>::iterator itend;

	};
}
#endif //_BRUSH_ENTRIES_H