#ifndef _CACHE_MANAGER_H
#define _CACHE_MANAGER_H

#include <list>
#include <string>
#include <zthread/Mutex.h>
#include <ServedItemVisitorImpl.h>
#include <log/log.h>
#include <ILoad.h>
#include <fstream>

//Forward References
template <class T>
class Iterator;
class Timer;
namespace cache
{	
	//Forward References
	class Mapper;
	class CacheIterator;
	class ICacheEntry;
	class IReplacementVisitor;
	struct Record;
	class Loader;
	class QueryFactory;
	class ConnectionFactory;
	class Connection;
	class Statement;
	class PerformanceStatistics;
	class cache::Brush;
	class Cache;
	class ReplacementPolicy;
	class AbstractRegionFactory;
	class ServedItemListener;
	class BrushVisitor;
	class AbstractQueryFactory;
	class QueryModifier;
	class IFragThresholdVisitor;

	using namespace std;

	class CacheManager : public Iload
	{
	public:
		CacheManager();
		CacheManager(Loader *loader,
			Cache* cache,
			ReplacementPolicy *replacementPolicy,
			AbstractQueryFactory *queryFactory,
			unsigned int fragmentationThreshold=5);

		CacheIterator* load(Brush &brush,
			IReplacementVisitor* replacementVisitor,
			IFragThresholdVisitor* fragThresholdVisitor,
			AbstractRegionFactory* regionFactory,
			PerformanceStatistics& perStatistics);//Returns the number of hits in the cache

		/*
		 *	This version of the call is used by the right now prefetcher.
		 *  This is used in case we want to make sure that the nodes in the cache are
		 *  fetched in a particular order. May not be a good design decision.
		 */
		CacheIterator* load(Brush &brush,
			IReplacementVisitor* replacementVisitor,
			IFragThresholdVisitor* fragThresholdVisitor,
			AbstractRegionFactory* regionFactory,
			PerformanceStatistics& performanceStatistics,
			QueryModifier& queryModifier);
		


		//Used only by the iterator
	public:	 
		void loadLeftOver();

		/**
		* Deletes the record Iteration process. Should work
		* even if all the objects are not read from the 
		* iterator
		*/
		void endLoad(unsigned long count);


		//load tuples regions until the cache is full.
		//For each new entry
		//a) remove one entry from the cache
		//b) Add a new entry into the cache
		//c) Store the pointer in the queue	


		/**
		* Add the served Listener
		* \param *servedItemVisitor 
		*/
		void addServedItemListener(
			ServedItemListener *servedItemListener);


		virtual ~CacheManager();

		/*
		 * This returns the time taken to search in the cache
		 * in msec
		 *	@return unsigned long
		 */
		unsigned long getSearchTime();


	private:
		//Member Functions
		/*
		* This function updates the state of the replacement policy to notify it of the hits
		* in the cache. Also it inserts the cache hits into the cache entry queue. This will work 
		* for the cache.
		*/
		void handleHits(std::list<ICacheEntry*>& listCacheEntry);

		//Members Varaibles 
		Cache* cache;
		ReplacementPolicy* replacementPolicy;
		cache::Brush currentBrush;
		Loader* loader;		
		std::queue<ICacheEntry*>* cacheEntryQueue;
		Iterator<Record*> *recordIterator;		
		BrushVisitor *brushVisitor;		
		IReplacementVisitor* replacementVisitor;
		//Stores the reference to the region factory.
		AbstractRegionFactory* regionFactory;
		//Stores a reference to the mutex used to make the class thread safe
		ZThread::Mutex *mutex;
		//Stores the object
		ServedItemVisitorImpl servedItemVisitorImpl;
		//Stores the logger object
		debug::log::CMessageGroup cacheLogger;
		//Stores the referenced to the query factory
		AbstractQueryFactory *queryFactory;		
		///////////////Used to evaluate the search time//////////////////
		//Statistics for evaluation purposesa
		Timer* timer;
		unsigned long searchTime;
		//////////////////////////////////////////////////////////////////
		//Used only for debugging purposes
		unsigned int callNumber;
		std::ofstream debugStream;
		std::string currentQuery;
		//private
		unsigned int fragmentationThreshold;
		unsigned long numberOfTuplesLoaded;
	};
}
#endif //_CACHE_MANAGER_H
