#ifndef _DISTANCE_CONTROLLER_H
#define _DISTANCE_CONTROLLER_H

#include <list>
#include <cache/CacheEntryIterator.h>
#include <cache/DistanceBrushLoader.h>


namespace cache
{
	class Brush;
	class BrushEntries;	
	class IIndex;

	class DistanceController
	{
public:
	DistanceController(IIndex *index,
					   unsigned long sizeOfBuffer);

	/*
	* Register a new brush with the brush controller
	*/
	void addBrush(Brush *newBrush);
	
	/*
	 *	Returns a pointer to ICacheEntry*
	 */
	ICacheEntry* peek();
	
	/*
	 *	Removes the entry for the ICacheEntry
	 */
	void remove();
	/*
	 * Returns a cache entry iterator
	 * The procedure does its best to order the entries as per
	 * the distance from the current brush
	 */
	CacheEntryIterator* getCacheEntryIterator(Brush *currentBrush);

	/*
	*	Selects the most distant brush
	*/
	//void selectDistance(Brush* currentBrush);
	
	/*
	 *	Selects the most distant brush and sets the values of
	 *  brushstore and replaeBrush accordingly
	 */
	void selectDistanceBrush(Brush *currentBrush);

	/**
	* Calculates the distance between the current and the candidate brush
	* \param currentBrush 
	* \param candidateBrush 
	* \return double
	*/
	double calculateDistance(Brush* currentBrush,
							 Brush* candidateBrush);

	
	/*
	 *	Clears the state of the DistanceController
	 * The currentBrush,currentBrushEntries etc..
	 */
	void clearState();

	~DistanceController();

	/**
	* Removes the brush entries
	*/
	void removeBrushEntries();

	/**
	* Makes a new BrushEntries
	* \param *brush 
	*/
	void makeBrushEntry(Brush *brush);

	/**
	* Returns the number of distinct nodes that brush store has track of
	* \return 
	*/
	unsigned long getOccupiedCount();

private:

	void saveLeftOverBrush();
	//Stores the list of brushes
	std::list<Brush*>* brushStore;
	//Used to store the brushes that have been iterated over
	std::list<Brush*>* tempBrushStore;
	//Store a reference to the loader
	DistanceBrushLoader distLoader;
	//Stores a reference to the currentActiveBrushEntries being replaced
	BrushEntries* brushEntries;
	//Stores a reference to the current Active brush replaced
	Brush* currentBrush;
	//Stores the size of the buffer
	unsigned long sizeOfBuffer;

};

}
#endif //_DISTANCE_CONTROLLER_H