#ifndef _DISTANCE_REPLACEMENT
#define _DISTANCE_REPLACEMENT

#include <BrushVisitor.h>
#include <ServedItemVisitor.h>
#include <ReplacementPolicy.h>
#include <Cache.h>
#include <CacheEntry.h>
#include <log/log.h>
#include <cache/DoublyList.h>
#include <cache/DistanceController.h>


namespace cache
{
	//Forward References
	class IIndex;
	class BrushEntries;
	class DistanceBrushStore;
	class CacheEntryIterator;
	class Brush;

	class DistanceReplacement : public ReplacementPolicy
	{
	public:		
		DistanceReplacement(Cache *cache,
							CacheEntry<bool>* buffer,
							unsigned long sizeOfBuffer,
							IIndex *index);		

		~DistanceReplacement();   

		ICacheEntry* getVictimItem();

		void newData(ICacheEntry &cacheEntry);

		void servedItem(ICacheEntry& cacheEntry);

		void hitCacheEntry(ICacheEntry& cacheEntry);

		void ifRemainder(bool flag);

		void brushModified(Brush& brush);

		void checkForError();

		ICacheEntry* peek();

		void clearCache();

		void removeEntries(std::list<ICacheEntry*>& listCacheEntry);

	private:
		//Stores the reference to the cache buffer
		CacheEntry<bool>* lBuffer;	
		//Stores the size of the buffer	
		unsigned long sizeOfBuffer;	
		//Store the reference to the cache
		Cache* cache;
		//Stores the index
		IIndex *index;
		//Stores the tempBrush
		Brush* tempBrush;
		//Stores the current brush
		Brush *currentBrush;
		//Stores the reference to the free list the free list stores the
		//group of free nodes available for replacement
		DoublyList<CacheEntry<bool>*>* freeList;
		//Stores a referenceo to the DistanceController
		DistanceController distController;
		//Stores a pointer to the current cacheEntryIterator
		CacheEntryIterator *cacheEntryIterator;
		//Sto
		ICacheEntry* peekResult;

	};
}
#endif