#ifndef _PREFETCHCONTROLLER_H
#define _PREFETCHCONTROLLER_H

#include <zthread/Thread.h>
#include <zthread/Mutex.h>
#include <zthread/Semaphore.h>
#include <IReplacementVisitor.h>
#include <iostream>
#include <PreEmptException.h>
#include <ICacheEntry.h>
#include <log/log.h>
	
namespace cache
{
	//Forward References
	class Prefetcher;
	/**
	*This class is responsible for starting and stoping the prefetcher
	*/

	class PrefetchController : public IReplacementVisitor
	{
	public:
		/**
		* Constrcuts a new Prefetch controller
		* \param *prefetcher 
		* The prefetchController deletes the prefetcher.
		* \return 
		*/
		PrefetchController(Prefetcher *prefetcher);


		//This will called in the context of the main thread
		void preempt();


		//This is also called in the context of the main thread
		void start();

		/*
		* This is used to get the load Permit just 
		* used for experiment purposes. And used only in the testRTree.cpp.
		* @Return ZThread::Semaphore
		*/
		ZThread::Semaphore* getLoadPerimit();



		/**
		*
		* \param *cacheEntry 
		* \return bool The return value decides whether the 
		* prefetcher should continure or not
		*/
		bool chosenItem(ICacheEntry &cacheEntry);


		~PrefetchController();

	private:
		//Stores the reference to the prefetcher
		Prefetcher *prefetcher;	
		//Stores the reference to the zthread
		ZThread::Thread* prefetchThread;
		//Stores the reference to the Task
		ZThread::Task* task;
		//Store the reference to the mutex
		ZThread::Mutex* mutex;
		//Stores the status of preemption
		bool preEmptFlag;
		//Stores the logger object
		debug::log::CMessageGroup prefetchControllerLogger;
		//Stores the Semaphore shared by the prefecher and the
		//PrefetchController
		ZThread::Semaphore* semPrefetchPermit;
		//Store the load permit
		ZThread::Semaphore* semLoadPermit;

	};
}
#endif // _PREFETCHCONTROLLER_H