// Dimension.h: interface for the Dimension class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __DIMENSION_H__
#define __DIMENSION_H__

#include <map>

typedef std::map<float, IValue*> ValueMap;
typedef ValueMap::iterator		 ValueMapIter;
typedef ValueMap::const_iterator ValueMapCIter;

class Dimension : public IDimension
{
public:
	// Structors
	Dimension();
	~Dimension();

	// Naming support
	const char*		getName() const;
	void			setName(const char *name);

	Type			getType() const;
	void			setType(Type t);
	
	IValue*			getMappedValue(float f) const;
	IDimension::MappingIterator*getMappingIterator();
	
	void			setMapping(float f, IValue *value);
	
	// compares two values of this dimension type to see if they are equal
	bool			equal();
	
	/// get the stream representation of this meta information
	bool			unmarshal(std::istream &is, ISerializable *parent);
	/// get the meta information from the stream representation
	bool			marshal(std::ostream& s, ISerializable *parent);
	
private:
	std::string mName;
	Type		mType;
	ValueMap	mMappedValues;

	class DimensionHandler;
	friend class DimensionHandler;
	DimensionHandler *mHandler;

//	class MappingIterator;
	friend class IDimension::MappingIterator;
};

#endif //__DIMENSION_H__
