#ifndef CLUSTER_H
#define CLUSTER_H

#include <xmdv/ARR.h>
#include <xmdv/okcdata.h>
#include <xmdv/brush.h>

#define IS_CONTOUR0  1<<0
#define IS_CONTOUR1  1<<1
#define IS_MAGNIFIED 1<<2
#define IS_BRUSHED   1<<3
#define IS_BRUSHED_ANY   1<<4

/*Once the number of children of a cluster lower than 
TREEMAP_LIMIT*number of data points of the data set, 
do not draw the details of this cluster on tree map.*/
#define TREEMAP_LIMIT 0.01
struct Cluster 
{
    int parent;//Reference to the parent in the cluster tree
    int num_children;//stores the number of children
    ARR<int> child;//Stores the index in the cluster tree for the child

    ARR<double> mean;//Stores the mean of each of the dimensions
//jing
    ARR<double> max;//Stores the max for each dimension
    ARR<double> min;//Stores the min of each dimension
//jing
    ARR<double> all_mean;

    okcdata::mark_type mark;

    // cfentry of a cluster
    int entries;
    ARR<double> sx; //Stores the actual values of dimensions of the cluster
    double sxx;
    double sqrt_radius; //Stores the radius
    double sqrt_diameter;

    double color;
    
    // The dimensions for the rect in tree-map
    double xmin;
    double xmax;
    double ymin;
    double ymax;

    int flags;    
    //jing_flat_sb_start
    bool flat_brushed;
    bool contain_flat_brushed;
    //jing_flat_sb_end

	Cluster() {}
private:
	Cluster(const Cluster&);
//	Cluster operator = (const Cluster&);
};

struct ClusterTree
{
    char *clusterfile;
    
    ARR<Cluster> nodes;
    int dimension;
    int treemap_limit_number;
    
    ARR<double> min_extents;
    ARR<double> max_extents;

    ClusterTree() :
        clusterfile(0),
        dimension(0)
    {}

    ~ClusterTree() {
		if (clusterfile) { delete [] clusterfile; clusterfile = NULL; }
		for (int i=0; i<nodes.Len(); ++i) {
			nodes[i].child.Clear();
			nodes[i].mean.Clear();
			nodes[i].max.Clear();
			nodes[i].min.Clear();
			nodes[i].all_mean.Clear();
			nodes[i].sx.Clear();
		}
		nodes.Clear();
		min_extents.Clear();
		max_extents.Clear();
    }   

    bool ReadClusterTreeFile(const char* fname);
    bool ReadBinaryClusterTreeFile(const char* fname);
	bool ReadClusterTreeFileWithDatabase(const char* fname);
    bool ReadClusterTreeFile2();
    bool ReadClusterTreeFile3();
};

extern int dep_dim;

extern void hierDeleteClusterTree();
extern void hierInitExtents();
extern void hierInitClusterTree();
extern void hierClearNodeFlags(int bits=0xffffff);
extern void hierMarkContourNodes(double radius, int FLAG);
extern void hierModifyExtentsByBrushParameters();
extern void hierModifyBrushParametersByExtents();
extern void hierMarkBrushedClustersByColor(Cluster* node, double L, double R);
extern void hierMarkBrushedClustersByExtents(Cluster* node);
extern void hierSetMeanOnOff();
extern void hierReorderDimension();

extern void hierMarkFlatBrushedClustersByExtents(Brush *brush, Cluster* node);
extern void HierChangeFlatBrushAccordingToClusters();
extern void JoinBrush( const Cluster &node, ARR<double> &brush_min, ARR<double> &brush_max);
extern void TraceJoinFSBBrushedWithClusters( const Cluster &node, ARR<double> &brush_min, ARR<double> &brush_max);


extern void SetupBrushParameters();

#endif
