///////////////////////////////////////////////////////////////////////////////
/// \file SettingsWindow.cpp
/// \brief Implementation of the CSettingsWindow class
/// \date 8/16/2004
/// \author Anilkumar Patro
///////////////////////////////////////////////////////////////////////////////

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include <xmdv/ogl.h>
#include <xmdv/FlatPixelSettings.h>
#include <xmdv/CFlatPixel.h>

#include <glgooey/BoxLayouter.h>
#include <glgooey/FlowLayouter.h>
#include <glgooey/GridLayouter.h>

using namespace Gooey;

// DEFINES
//

// FORWARD REFERENCES
//

//-----------------------------------------------------------------------------
CFlatPixelSettings::CFlatPixelSettings(Tcl_Interp *interp)
	: FrameWindow(Core::Rectangle(10, 350, 700, 520), 0, "Toolbox"),
	  m_mainpanel(Core::Rectangle(10, 10, 20, 20), 0),
	  m_horizpanel1(Core::Rectangle(10, 10, 20, 20), 0),
//	  m_horizpanel11(Core::Rectangle(10, 10, 20, 20), 0),
//	  m_horizpanel12(Core::Rectangle(10, 10, 20, 20), 0),
	  m_horizpanel2(Core::Rectangle(10, 10, 20, 20), 0),
//	  m_horizpanel3(Core::Rectangle(10, 10, 20, 20), 0),
//	  m_horizpanel31(Core::Rectangle(10, 10, 20, 20), 0),
//	  m_horizpanel32(Core::Rectangle(10, 10, 20, 20), 0),
	  m_btnColors(0, "Color Requester"), m_btnWeights(0, "Change Weights"),
	  m_btnBrushToolbox(0, "Brush Toolbox"),
	  m_btnBrushedVals(0, "Brushed Values"),
	  m_btnDataStats(0, "Data Statistics"),
	  m_lblSortedBy(0, "Sorted by:"),
	  m_cmbSortedBy(0, Core::Rectangle(10, 10, 20, 20)),
	  m_lblOrder(0, "Order:"),
	  m_cmbSortOrder(0, Core::Rectangle(10, 10, 20, 20)),
	  m_lblSelect(0, "Selected Dim:"),
	  m_cmbSelectedDim(0, Core::Rectangle(10, 10, 20, 20)),
	  m_sliDistortion(0, Slider::horizontal, Core::Rectangle(10, 10, 20, 20)),
	  m_chkShowColormap(0, "Show Colormap"),
	  m_chkShowDerived(0, "Show Derived Dim"),
	  m_chkMDSPlacement(0, "MDS Placement"), 
	  m_chkSampling(0, "Sampling"),
	  m_sliSampling(0, Slider::horizontal, Core::Rectangle(10, 10, 20, 20))
{
	m_mainpanel.setLayouter(new BoxLayouter(BoxLayouter::vertical, 5.0f, false));
	m_horizpanel1.setLayouter(new GridLayouter(1,7, 10, 10));
	m_horizpanel2.setLayouter(new GridLayouter(2, 5));

	m_cmbSortedBy.selectionChanged.connect(this, &CFlatPixelSettings::OnSelChanged_cmbSortedBy);
	m_cmbSortOrder.selectionChanged.connect(this, &CFlatPixelSettings::OnSelChanged_cmbSortOrder);
	m_cmbSortOrder.addString("Ascending");
	m_cmbSortOrder.addString("Descending");
	m_horizpanel1.addChildWindow(&m_lblSortedBy);
	m_horizpanel1.addChildWindow(&m_cmbSortedBy);
	m_horizpanel1.addChildWindow(&m_lblOrder);
	m_horizpanel1.addChildWindow(&m_cmbSortOrder);

	m_cmbSelectedDim.selectionChanged.connect(this, &CFlatPixelSettings::OnSelChanged_cmbSelectedDim);
	m_sliDistortion.valueChanged.connect(this, &CFlatPixelSettings::OnValueChanged_sliDistortion);
	m_sliDistortion.setMinimum(0);
	m_sliDistortion.setMaximum(100);
	m_horizpanel1.addChildWindow(&m_lblSelect);
	m_horizpanel1.addChildWindow(&m_cmbSelectedDim);
	m_horizpanel1.addChildWindow(&m_sliDistortion);

	m_btnColors.pressed.connect(this, &CFlatPixelSettings::OnClicked_btnColors);
	m_btnWeights.pressed.connect(this, &CFlatPixelSettings::OnClicked_btnWeights);
	m_btnBrushToolbox.pressed.connect(this, &CFlatPixelSettings::OnClicked_btnBrushToolbox);
	m_btnBrushedVals.pressed.connect(this, &CFlatPixelSettings::OnClicked_btnBrushedVals);
	m_btnDataStats.pressed.connect(this, &CFlatPixelSettings::OnClicked_btnDataStats);
	m_horizpanel2.addChildWindow(&m_btnColors);
	m_horizpanel2.addChildWindow(&m_btnWeights);
	m_horizpanel2.addChildWindow(&m_btnBrushToolbox);
	m_horizpanel2.addChildWindow(&m_btnBrushedVals);
	m_horizpanel2.addChildWindow(&m_btnDataStats);

	m_chkShowColormap.setMouseOverStatus(true);
	m_chkShowColormap.pressed.connect(this, &CFlatPixelSettings::OnClicked_chkShowColormap);
	m_chkShowDerived.setMouseOverStatus(true);
	m_chkShowDerived.pressed.connect(this, &CFlatPixelSettings::OnClicked_chkShowDerived);
	m_chkMDSPlacement.pressed.connect(this, &CFlatPixelSettings::OnClicked_chkMDSPlacement);
	m_chkSampling.pressed.connect(this, &CFlatPixelSettings::OnClicked_chkSampling);

	m_horizpanel2.addChildWindow(&m_chkShowColormap);
	m_horizpanel2.addChildWindow(&m_chkShowDerived);
	m_horizpanel2.addChildWindow(&m_chkMDSPlacement);

	m_sliSampling.setMinimum(0);
	m_sliSampling.setMaximum(100);
	m_sliSampling.setValue(50);
	m_sliSampling.disable();
	m_sliSampling.valueChanged.connect(this, &CFlatPixelSettings::OnValueChanged_sliSampling);
	m_horizpanel2.addChildWindow(&m_chkSampling);
	m_horizpanel2.addChildWindow(&m_sliSampling);

	m_mainpanel.addChildWindow(&m_horizpanel1);
	m_mainpanel.addChildWindow(&m_horizpanel2);

	m_horizpanel1.arrangeChildren();
	m_horizpanel2.arrangeChildren();
	m_mainpanel.arrangeChildren();

	setClientWindow(&m_mainpanel);
	disableMovement();
	m_interp = interp;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
CFlatPixelSettings::~CFlatPixelSettings()
{
	// TODO: Add destructor
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/*virtual*/ void CFlatPixelSettings::onShow()
{
	m_cmbSortedBy.removeAllStrings(); m_cmbSelectedDim.removeAllStrings();
	m_cmbSortedBy.addString("Combined Distance");
	m_cmbSelectedDim.addString("Combined Distance");
	for (int i = 0; i < g_globals.dataset->dims; ++i) {
		m_cmbSortedBy.addString(g_globals.dataset->names[i]);
		m_cmbSelectedDim.addString(g_globals.dataset->names[i]);
	}

	int selDim = g_globals.flatqipixel->GetSelectedDim();
	m_cmbSelectedDim.selectStringAt(selDim);
	m_sliDistortion.setValue((int)(g_globals.flatqipixel->GetDimDistortion(selDim)*100.0f));

	m_cmbSortedBy.selectStringAt(g_globals.flatqipixel->GetSortedDim());
	m_cmbSortOrder.selectStringAt(g_globals.flatqipixel->GetSortDimOrder() == true ? 0 : 1);

	return Gooey::FrameWindow::onShow();
}
//-----------------------------------------------------------------------------

#define WINDOW_HEIGHT	180

//-----------------------------------------------------------------------------
void CFlatPixelSettings::Update(int w, int h)
{
	setPosition(Core::Vector2(0.0f, (float) (h - WINDOW_HEIGHT)));
	setSize(Core::Vector2((float) w, WINDOW_HEIGHT));
}
//-----------------------------------------------------------------------------

void CFlatPixelSettings::Reset()
{
	if (isVisible())
		onShow();
}

void CFlatPixelSettings::SetSelectedDim(int dim)
{
	if (isVisible()) {
		m_cmbSelectedDim.selectStringAt(dim);
	}
}

void CFlatPixelSettings::SetSortedByDim(int dim)
{
	if (isVisible()) {
		m_cmbSortedBy.selectStringAt(dim);
	}
}

void CFlatPixelSettings::SetSortingOrder(bool asc)
{
	if (isVisible()) {
		m_cmbSortOrder.selectStringAt((asc == true) ? 0 : 1);
	}
}

void CFlatPixelSettings::OnClicked_btnColors()
{
	Tcl_Eval(m_interp, "ShowColorRequesterDialog");
}

void CFlatPixelSettings::OnClicked_btnWeights()
{
	Tcl_Eval(m_interp, "ShowPixWeightsDialog");
//	Tcl_Eval(m_interp, "Window show [W PixWeightsDialog]");
//	Tcl_Eval(m_interp, "raise [W PixWeightsDialog]");
//	Tcl_Eval(m_interp, "::.pixWeights::SetupWeightsCanvas");
}

void CFlatPixelSettings::OnClicked_btnBrushToolbox()
{
	Tcl_Eval(m_interp, "ShowPixBrushDialog");
}

void CFlatPixelSettings::OnClicked_btnBrushedVals()
{
	Tcl_Eval(m_interp, "ShowDataValuesDialog");
}

void CFlatPixelSettings::OnClicked_btnDataStats()
{
	Tcl_Eval(m_interp, "ShowDataSummaryDialog");
}

void CFlatPixelSettings::OnClicked_chkShowColormap()
{
	if (g_globals.flatqipixel->IsShowingColormap())
		g_globals.flatqipixel->ShowColormap(false);
	else
		g_globals.flatqipixel->ShowColormap(true);
}

void CFlatPixelSettings::OnClicked_chkShowDerived()
{
	if (g_globals.flatqipixel->IsShowingDerived())
		g_globals.flatqipixel->ShowDerived(false);
	else
		g_globals.flatqipixel->ShowDerived(true);
}

void CFlatPixelSettings::OnClicked_chkMDSPlacement()
{
	if (g_globals.flatqipixel->GetPlacementType() == CFlatPixel::PPT_NORMAL)
		g_globals.flatqipixel->SetPlacementType(CFlatPixel::PPT_MDS);
	else
		g_globals.flatqipixel->SetPlacementType(CFlatPixel::PPT_NORMAL);
}

void CFlatPixelSettings::OnSelChanged_cmbSortedBy(const std::string& newDim)
{
	g_globals.flatqipixel->SetSortedDim(m_cmbSortedBy.selectedIndex());
}

void CFlatPixelSettings::OnSelChanged_cmbSortOrder(const std::string& newOrder)
{
	g_globals.flatqipixel->SetSortDimOrder(m_cmbSortOrder.selectedIndex() == 0);
}

void CFlatPixelSettings::OnSelChanged_cmbSelectedDim(const std::string& newSelectedDim)
{
	int selDim = m_cmbSelectedDim.selectedIndex();
	g_globals.flatqipixel->SelectDim(selDim);
	m_sliDistortion.setValue((int)(g_globals.flatqipixel->GetDimDistortion(selDim)*100.0f));
}

void CFlatPixelSettings::OnValueChanged_sliDistortion(int value)
{
	g_globals.flatqipixel->SetDimDistortion(
		g_globals.flatqipixel->GetSelectedDim(), value/100.0f);
}

void CFlatPixelSettings::OnClicked_chkSampling()
{
	if (m_sliSampling.isEnabled()) {
		m_sliSampling.disable();
		g_globals.flatqipixel->SetSamplingFactor(1.0f);
	}
	else {
		m_sliSampling.enable();
		g_globals.flatqipixel->SetSamplingFactor(m_sliSampling.value()/100.0f);
	}
}

void CFlatPixelSettings::OnValueChanged_sliSampling(int value)
{
	g_globals.flatqipixel->SetSamplingFactor(value/100.0f);
}

//-----------------------------------------------------------------------------
PlusMinusPanel::PlusMinusPanel(const Gooey::Core::Rectangle &aRectangle,
							   Gooey::Window *window)
	: Gooey::Panel(aRectangle, 0, "Simple"), m_btnShowWindow(0, "+")
{
	m_theWindow = window;

	m_btnShowWindow.addString("-");
	m_btnShowWindow.setSize(Gooey::Core::Vector2(20, 20));
	m_btnShowWindow.pressed.connect(this, &PlusMinusPanel::OnShowInfo_Pressed);

	setLayouter(new Gooey::FlowLayouter(Gooey::FlowLayouter::center));
	addChildWindow(&m_btnShowWindow);
	arrangeChildren();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void PlusMinusPanel::OnShowInfo_Pressed()
{
	if (m_theWindow->isVisible())
		m_theWindow->hide();
	else
		m_theWindow->show();
}
//-----------------------------------------------------------------------------
