///////////////////////////////////////////////////////////////////////////////
/// \file PixelTool.cpp
/// \brief Implementation of the CPixelTool class
/// \date 7/5/2004
/// \author Anilkumar Patro
///////////////////////////////////////////////////////////////////////////////

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include <xmdv/PixelTool.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/CFlatDisplayInformation.h>
#include <xmdv/ogl.h>

// DEFINES
//

// FORWARD DECLARATION
//

//-----------------------------------------------------------------------------
CPixelOriented::MarkerStatus CPixelTool::Marker::contains(float x,float y)
{
	if (x >= x1 && x <= x2 /*&& y >= y1 && y <= y2*/) {
		double brushMax, brushMin;
		GetNormalizedBrushExtents(&g_globals.brushes[0], dim, brushMax, brushMin);

		double lo	= YMAP(brushMin);
		double hi	= YMAP(brushMax);
		if ((hi - y) > (y - lo))
			return MST_ON_MAX;
		else
			return MST_ON_MIN;
	}
	return MST_BLANK;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
CPixelTool::CPixelTool(CFlatDisplayInformation *flat_info)
	: CFlatPixel(flat_info)
{
	m_currDim = -1;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
CPixelTool::~CPixelTool()
{
	// TODO: Add destructor
}
//-----------------------------------------------------------------------------

//-------------------------------------------------------------------------
CPixelTool & CPixelTool::operator =(const CPixelTool &from)
{
	if (this == &from)
		return *this;

	// TODO: Add assignment operator

	return *this;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CPixelTool::Draw()
{
	if (!g_globals.dataset)
		return;

	//set canvas background, clear canvas
	PrepareCanvas();

	if (m_markers.size() != 2 * g_globals.dataset->dims)
	{
		m_markers.clear();
		for (int i = 0; i < 2 * g_globals.dataset->dims; ++i)
			m_markers.push_back(Marker((int)(i/2)));
	}

	Vec2 font_size	= canvas->GetDataFontSize("W");
	double max_str_size;

	float x			= 0.0f;
	//modified by Zaixian, changed 0.3f to g_globals.dataset->one_over_dims
	for (int dim = 0; dim < g_globals.dataset->dims; ++dim, x += (float)g_globals.dataset->one_over_dims)
	{
		double brushMax, brushMin;
		GetNormalizedBrushExtents(&g_globals.brushes[0], dim, brushMax, brushMin);

		// draw the colormap
		g_globals.pixelBrushedCMap->DrawColorBar(x, (float) YMAP(brushMax), x,
												 (float) YMAP(brushMin), 20.0f);
		g_globals.pixelUnbrushedCMap->DrawColorBar(x, (float) YMAP(brushMin), x,
												   1.0f, 20.0f);
		g_globals.pixelUnbrushedCMap->DrawColorBar(x, (float) YMAP(brushMax), x,
												   0.0f, 20.0f);

		// draw markers
		glBegin(GL_TRIANGLES); {
			MarkerStatus markerStatus	= m_markers[2 * dim].status;
			if (markerStatus == MST_ON_MIN || markerStatus == MST_DRAG_MIN)
				glColor3f(1.0f, 0, 0);
			else
				glColor3f(0, 0, 0);
			m_markers[2 * dim].setExtents(x + .02f, (float)YMAP(brushMin) - .02f,
										  x + .04f, (float)YMAP(brushMin) + .02f);
			glVertex2f(x + .02f, (float)YMAP(brushMin));
			glVertex2f(x + .04f, (float)YMAP(brushMin) + 0.02f);
			glVertex2f(x + .04f, (float)YMAP(brushMin) - 0.02f);

			markerStatus = m_markers[2 * dim + 1].status;
			if (markerStatus == MST_ON_MAX || markerStatus == MST_DRAG_MAX)
				glColor3f(1.0f, 0, 0);
			else
				glColor3f(0, 0, 0);
			m_markers[2 * dim + 1].setExtents(x + .02f, (float)YMAP(brushMax) - .02f,
											  x + .04f, (float)YMAP(brushMax) + .02f);
			glVertex2f(x + .02f, (float)YMAP(brushMax));
			glVertex2f(x + .04f, (float)YMAP(brushMax) + 0.02f);
			glVertex2f(x + .04f, (float)YMAP(brushMax) - 0.02f);
		} glEnd();

		// draw names, min and max
		if (dim != g_globals.dataset->dims - 1)
			max_str_size = g_globals.screen_distortion->paracoord_interval[dim] -
						   font_size.X * 5;
		else
			max_str_size = g_globals.screen_distortion->paracoord_interval[dim]; 
		canvas->DrawConfinedString(x, font_size.Y,
								   g_globals.dataset->names[dim], max_str_size);

		// Draw the MAX2 values of the dimension
		canvas->DrawConfinedString(x, -.5 * font_size.Y,
								   g_globals.dataset->GetMax(dim), max_str_size);

		// Draw the MIN2 values of the dimension
		canvas->DrawConfinedString(x, 1.0 - font_size.Y,
								   g_globals.dataset->GetMin(dim), max_str_size);
	}
}
//-----------------------------------------------------------------------------

static double resize_y;
//-----------------------------------------------------------------------------
void CPixelTool::InputMouseMotion(int sx, int sy)
{
	if (!g_globals.dataset)
		return;

	// get brush extents
	Vec2 p	= canvas->MapScreenToData(Vec2(sx, sy));

	//	ShowCaption(canvas, sx, sy);

	if (canvas->button[2] == true)
		appCanvasMousePan(canvas);

	if (m_currDim >= 0)
	{
		int dim				= m_currDim;
		Marker &markerMin	= m_markers[2 * dim];
		Marker &markerMax	= m_markers[2 * dim + 1];


		// Clamp y coordinate to within dimension limits
		double y			= YINVMAP(dim, p.Y);
		double move_y		= MAX2(y, g_globals.dataset->dim_min[dim]);
		move_y = MIN2(move_y, g_globals.dataset->dim_max[dim]);

		if (markerMin.status == MST_RESIZE)
		{
			double diff_y	= y - resize_y;
			double sz		= g_globals.brushes[0].Size(dim);
			double pos		= g_globals.brushes[0].Pos(dim);
			double newsz;
			if (diff_y > 0)
				newsz = sz * 1.1;
			else
				newsz = sz * 0.9;

			double hy	= MIN2(pos + newsz*.5, g_globals.dataset->dim_max[dim]);
			double ly	= MAX2(pos - newsz*.5, g_globals.dataset->dim_min[dim]);
			g_globals.brushes[0].SetMinMax(dim, ly, hy);
//			g_globals.dataset->ComputeQueryDistances(
//				g_globals.flatqipixel->GetSortedDim(),
//				g_globals.flatqipixel->GetSortDimOrder());
			resize_y = y;
		} else if (markerMin.status == MST_TRANSLATE)
		{
			double trans_y	= MIN2(y,
								   g_globals.dataset->dim_max[dim] -
								   g_globals.brushes[0].Size(dim) * .5);
			trans_y = MAX2(trans_y,
						   g_globals.dataset->dim_min[dim] +
						   g_globals.brushes[0].Size(dim) * .5);

			double ly	= (dim, trans_y - g_globals.brushes[0].Size(dim) * .5);
			double hy	= (dim, trans_y + g_globals.brushes[0].Size(dim) * .5);
			g_globals.brushes[0].SetMinMax(dim, ly, hy);
//			g_globals.dataset->ComputeQueryDistances(
//				g_globals.flatqipixel->GetSortedDim(),
//				g_globals.flatqipixel->GetSortDimOrder());
		} else if (markerMin.status == MST_DRAG_MIN)
		{
			double hy	= g_globals.brushes[0].Max(dim);
			double ly	= MIN2(move_y, hy);
			g_globals.brushes[0].SetMinMax(dim, ly, hy);
//			g_globals.dataset->ComputeQueryDistances(
//				g_globals.flatqipixel->GetSortedDim(),
//				g_globals.flatqipixel->GetSortDimOrder());
		} else if (markerMax.status == MST_DRAG_MAX)
		{
			double ly	= g_globals.brushes[0].Min(dim);
			double hy	= MAX2(move_y, ly);
			g_globals.brushes[0].SetMinMax(dim, ly, hy);
//			g_globals.dataset->ComputeQueryDistances(
//				g_globals.flatqipixel->GetSortedDim(),
//				g_globals.flatqipixel->GetSortDimOrder());
		}
		appBrushBoundaryCaption(canvas, m_currDim);
		appCanvasBrushHasChanged(Togl_Interp(canvas->GetWidget()));
		appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
	} else
		//modified by Zaixian, changed all_dims to dims
		for (int dim = 0; dim < g_globals.dataset->dims; ++dim)
		{
			Marker &markerMin	= m_markers[2 * dim];
			Marker &markerMax	= m_markers[2 * dim + 1];

			if (markerMin.status != MST_DRAG_MIN &&
				markerMax.status != MST_DRAG_MAX)
			{
				// check if mouse is on markers
				markerMin.status = markerMin.contains((float)p.X, (float)p.Y);
				markerMax.status = markerMin.contains((float)p.X, (float)p.Y);
				canvas->Refresh();
			}
		}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CPixelTool::InputButtonPress(int button, int sx, int sy)
{
	if (!g_globals.dataset)
		return;

	Vec2 cursor	= canvas->MapScreenToData(Vec2(sx, sy));

	if (button == 2)
	{
		canvas->SetHandCursor();
		//		isPanning = true;
		return;
	}

	float x	= 0.0f;
	//modified by Zaixian, changed 0.3f to g_globals.dataset->one_over_dims
	for (int dim = 0; dim < g_globals.dataset->dims; ++dim, x += (float)g_globals.dataset->one_over_dims)
	{
		Marker &markerMin	= m_markers[2 * dim];
		Marker &markerMax	= m_markers[2 * dim + 1];

		if (button == 0)
		{
			if (markerMin.status == MST_ON_MIN)
			{
				markerMin.status = MST_DRAG_MIN;
				m_currDim = dim;
				break;
			} else if (markerMax.status == MST_ON_MAX)
			{
				markerMax.status = MST_DRAG_MAX;
				m_currDim = dim;
				break;
			}
		}
		double brushMax, brushMin;
		GetNormalizedBrushExtents(&g_globals.brushes[0], dim, brushMax, brushMin);

		if (cursor.X >= x - .02f && cursor.X <= x + .02f)
		{
			if (cursor.Y <= YMAP(brushMin) && cursor.Y >= YMAP(brushMax))
			{
				if (button == 0)
				{
					markerMin.status = MST_TRANSLATE;
					m_currDim = dim;
					break;
				} else if (button == 1)
				{
					markerMin.status = MST_RESIZE;
					resize_y = YINVMAP(dim, cursor.Y);
					m_currDim = dim;
					break;
				}
			}
		}
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CPixelTool::InputButtonRelease(int button, int x, int y)
{
	if (canvas->shiftkey)
		return;

	if (m_currDim >= 0)
	{
		Marker &markerMin	= m_markers[2 * m_currDim];
		Marker &markerMax	= m_markers[2 * m_currDim + 1];

		if (markerMin.status == MST_DRAG_MIN ||
			markerMax.status == MST_DRAG_MAX ||
			markerMin.status == MST_TRANSLATE ||
			markerMin.status == MST_RESIZE)
		{
			markerMin.status = MST_BLANK;
			markerMax.status = MST_BLANK;
			m_currDim = -1;
		}
	}

	canvas->SetDefaultCursor();
	appCanvasBrushHasChanged(Togl_Interp(canvas->GetWidget()));
	appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
}
//-----------------------------------------------------------------------------
