
// SYSTEM INCLUDES
//
#include <string>
#include <cmath>
#include <fstream>
#include <sstream>
using namespace std;

// PROJECT INCLUDES
//
#include <xmdv/SimilarityCalculator.h>

CDBSimilarityCalculator :: CDBSimilarityCalculator()
{
/*	numDims = 0;
    numBinRows = 0;
    numBins = 1;
    numDataItems = 0;
*/    bins = NULL;
    similarityMatrix = NULL;
}

CDBSimilarityCalculator :: CDBSimilarityCalculator(int numDims, int numBins)
{
    bins = NULL;
    similarityMatrix = NULL;

    init(numDims, numBins);
}

//CDBSimilarityCalculator :: CDBSimilarityCalculator(int numDims, int numBins, CDBManager* db, string tableName)
//{
//    bins = NULL;
//    similarityMatrix = NULL;
//
//    init(numDims, numBins);
//
//	GetBinsFromDBSimilarityTable(db, tableName);
//    CalculateSimilarityMatrix2();
//}

CDBSimilarityCalculator :: ~CDBSimilarityCalculator()
{
    int i;

    if (bins != NULL)
    {
        for (i = 0; i < numBinRows; i++)
        {
            delete [] bins[i];
        }
        delete bins;
    }

    if (similarityMatrix != NULL)
    {
        for (i = 0; i < numDims; i++)
        {
            delete [] similarityMatrix[i];
        }
        delete similarityMatrix;
    }
}

bool CDBSimilarityCalculator :: init(int numDims, int numBins)
{
    numDataItems = 0;
    //set numDims and numBins;
    if (numDims < 1)
        return false;
    if (numBins < 1)
        return false;
    this->numDims = numDims;
    this->numBins = numBins;
    this->numBinRows = (numDims+1)*numDims/2;

    int i, j, k;

    //init bins;
    bins = new double*[numBinRows];
    for (i = 0; i < numBinRows; i++)
    {
        bins[i] = new double[numBins];
    }

    for (i = 0; i < numBinRows; i++)
    {
        for (k = 0; k < numBins; k++)
        {
            bins[i][k] = 0;
        }
    }

    //init similarityMatrix;
    similarityMatrix = new double*[numDims];
    for (i = 0; i < numDims; i++)
    {
        similarityMatrix[i] = new double[numDims];
    }

    for (i = 0; i < numDims; i++)
    {
        for (j = 0; j < numDims; j++)
        {
            similarityMatrix[i][j] = similarityMatrix[j][i] = 1;
        }
    }
    return true;
}

bool CDBSimilarityCalculator :: Update(double* values)
{
    int i, j;

    for (i = 0; i < numDims; i++)
    {
        for (j = i; j < numDims; j++)
        {
            int index = (int)(fabs(values[i] - values[j])*numBins);
            if (index >= numBins)
                index = numBins-1;
            bins[GetIndex(i,j)][index] += 1;
//            if (i != j)
//                bins[j][i][index] += 1;
        }
    }

    numDataItems += 1;
    return true;
}

//bool CDBSimilarityCalculator :: StoreSimilarityTableToDB(CDBManager* db, string tableName)
//{
//    if (bins == NULL)
//        return false;
//
//    //create table
//    db->ExecuteUpdate(GetDropTableString(tableName));
//    db->ExecuteUpdate(GetCreateTableString(tableName));
//
//    //load rows
//    int i;
//
//    for (i = 0; i < numBinRows; i++)
//    {
//        db->ExecuteUpdate(GetInsertTableString(tableName, bins[i]));
//    }
//
//    db->Commit();
//    return true;
//}
//
////note: you need call init() before call this function.
//bool CDBSimilarityCalculator :: GetBinsFromDBSimilarityTable(CDBManager* db, string tableName)
//{
//    if (bins == NULL)
//        return false;
//
//    //create query
//    ResultSet* re = db->ExecuteQuery("select * from " + tableName);
//
//    //load rows to bins
//    int i, k;
//
//    for (i = 0; i < numBinRows; i++)
//    {
//		if (!re->next())
//		{
//            delete re;
//			return false;
//		}
//
//		for (k = 0; k < numBins; k++)
//		{
//			bins[i][k] = re->getDouble(k);
//		}
//    }
//    delete re;
//
//    numDataItems = 0;
//    for (i = 0; i < numBins; i++)
//    {
//        numDataItems += bins[0][i];
//    }
//    return true;
//}

//threshold to handle outliers to be added
void CDBSimilarityCalculator :: CalculateSimilarityMatrix3()
{
    int i;
    int it = 10;
    double interval = 1.0/it;

    double thre = interval;
    double best_thre = thre;
    double max_var = 0;
    for (i = 1; i < it+1; i++)
    {
        CalculateSimilarityMatrix(thre);
        double var = CalculateDistanceVariance();
        if (var > max_var)
        {
            max_var = var;
            best_thre = thre;
        }
        
        thre += interval;
    }


    int i2;
    int it2 = 100;
    double interval2 = 1.0/it2;

    double thre2 = interval2;
    double best_thre2 = thre2;
    double max_var2 = 0;
    for (i2 = 1; i2 < it2/2; i2++)
    {
        CalculateSimilarityMatrix2(thre2);
        double var = CalculateDistanceVariance();
        if (var > max_var2)
        {
            max_var2 = var;
            best_thre2 = thre2;
        }
        
        thre2 += interval2;
    }

    if (max_var2 > max_var)
        CalculateSimilarityMatrix2(best_thre2);
    else
        CalculateSimilarityMatrix(best_thre);
}

//threshold to handle outliers to be added
void CDBSimilarityCalculator :: CalculateSimilarityMatrix()
{
    int i;
    int it = 10;
    double interval = 1.0/it;

    double thre = interval;
    double best_thre = thre;
    double max_var = 0;
    for (i = 1; i < it+1; i++)
    {
        CalculateSimilarityMatrix(thre);
        double var = CalculateDistanceVariance();
        if (var > max_var)
        {
            max_var = var;
            best_thre = thre;
        }
        
        thre += interval;
    }

    CalculateSimilarityMatrix(best_thre);

}

//threshold to handle outliers to be added
void CDBSimilarityCalculator :: CalculateSimilarityMatrix2()
{
    int i;
    int it = 100;
    double interval = 1.0/it;

    double thre = interval;
    double best_thre = thre;
    double max_var = 0;
    for (i = 1; i < it/2; i++)
    {
        CalculateSimilarityMatrix2(thre);
        double var = CalculateDistanceVariance();
        if (var > max_var)
        {
            max_var = var;
            best_thre = thre;
        }
        
        thre += interval;
    }

    CalculateSimilarityMatrix2(best_thre);
//    SmoothDistanceMatrix();
}

double CDBSimilarityCalculator :: CalculateDistanceVariance()
{
    int i, j;

    double ave = 0;
    int count = 0;
    for (i = 0; i < numDims; i++)
    {
        for (j = i+1; j < numDims; j++)
        {
            ave += similarityMatrix[i][j];
            count ++;
        }
    }
    ave /= count;

    double var = 0;
    for (i = 0; i < numDims; i++)
    {
        for (j = i+1; j < numDims; j++)
        {
            var += (ave - similarityMatrix[i][j]) * (ave - similarityMatrix[i][j]);
        }
    }

    return var;
}

void CDBSimilarityCalculator :: SmoothDistanceMatrix()
{
    int i, j;

    double ave = 0;
    int count = 0;
    for (i = 0; i < numDims; i++)
    {
        for (j = i+1; j < numDims; j++)
        {
            ave += similarityMatrix[i][j];
            count ++;
        }
    }
    ave /= count;

    double var = 0;
    for (i = 0; i < numDims; i++)
    {
        for (j = i+1; j < numDims; j++)
        {
            var += (ave - similarityMatrix[i][j]) * (ave - similarityMatrix[i][j]);
        }
    }

    var = sqrt(var/(count-1));

    double max_distance = ave + var;
    if (max_distance >= 1)
        return;

    for (i = 0; i < numDims; i++)
    {
        for (j = 0; j < numDims; j++)
        {
            if (similarityMatrix[i][j] > max_distance)
                similarityMatrix[i][j] = max_distance;
        }
    }
    
    return;
}

//threshold to handle outliers to be added
void CDBSimilarityCalculator :: CalculateSimilarityMatrix(double threshold)
{
    int i, j, k;

    int valid_num = (int)(threshold * numDataItems);
    int curr_num;
    for (i = 0; i < numDims; i++)
    {
        for (j = i; j < numDims; j++)
        {
            double sum = 0;
            int index = GetIndex(i,j);

            curr_num = 0;
            for (k = 0; k < numBins; k++)
            {
                if (curr_num < valid_num)
                {
                    curr_num += bins[index][k];
                    if (curr_num <= valid_num )
                        sum += (k) * (k) * bins[index][k];
                    else
                        sum += (k) * (k) * (bins[index][k] - curr_num + valid_num);
                }
            }
            similarityMatrix[i][j] = similarityMatrix[j][i] = sqrt(sum/valid_num) / numBins;
        }
    }

//    for (i = 0; i < numDims; i++)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
//        similarityMatrix[i][i] = 1;
}

//threshold to handle outliers to be added
void CDBSimilarityCalculator :: CalculateSimilarityMatrix2(double threshold)
{
    int i, j, k, m;

    int valid_bins = (int)(threshold * numBins);
    for (i = 0; i < numDims; i++)
    {
        for (j = i; j < numDims; j++)
        {
            double sum = 0;
            int index = GetIndex(i,j);

            int *temp = new int[numBins];
            
            for (k = 0; k < numBins; k++)
            {
                temp[k] = bins[index][k];
            }

            int total = 0;

            for (m = 0; m < valid_bins; m++)
            {
                int max = 0;
                int which = -1;
                for (k = 0; k < numBins; k++)
                {
                    if (temp[k] > max)
                    {
                        max = temp[k];
                        which = k;
                    }
                }

                if (which != -1)
                {
                    temp[which] = 0;
                    total += max;
                }
            }

            delete [] temp;

            similarityMatrix[i][j] = similarityMatrix[j][i] = 1 - ((double)total)/numDataItems;
        }
    }

//    for (i = 0; i < numDims; i++)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
//        similarityMatrix[i][i] = 1;
}

/*//threshold to handle outliers to be added
void CDBSimilarityCalculator :: CalculateSimilarityMatrix(double threshold)
{
    int i, j, k;

    for (i = 0; i < numDims; i++)
    {
        for (j = i; j < numDims; j++)
        {
            double sum = 0;
            int index = GetIndex(i,j);
            for (k = 0; k < numBins; k++)
            {
                sum += (k) * (k) * bins[index][k];
            }
            similarityMatrix[i][j] = similarityMatrix[j][i] = sqrt(sum/numDataItems) / numBins;
        }
    }

//    for (i = 0; i < numDims; i++)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
//        similarityMatrix[i][i] = 1;
}
*/

void CDBSimilarityCalculator :: GetSimilarityMatrix(double **matrix)
{
    int i, j;

    for (i = 0; i < numDims; i++)
    {
        for (j = i; j < numDims; j++)
        {
            matrix[i][j] = matrix[j][i] = similarityMatrix[i][j];
        }
    }
}


int CDBSimilarityCalculator :: GetSortDimension()
{
    int i, j;

    int sort_dim = -1;
    double min_total = 1000000;
    for (i = 0; i < numDims; i++)
    {
        double total = 0;
        for (j = 0; j < numDims; j++)
        {
            total += similarityMatrix[i][j];
        }

        if (total < min_total)
        {
            sort_dim = i;
            min_total = total;
        }
    }

    return sort_dim;
}


//string CDBSimilarityCalculator ::  GetCreateTableString(string tableName)
//{
//	ostringstream oss;
//	
//    oss << "create table " << tableName << "(";
//
//	int i;
//	for (i = 0; i < numBins-1; i++)
//	{
//		oss << "t" << i << " int,";
//	}
//    oss << "t" << i << " int)";
//
//	return oss.str();
//
//}
//
//string CDBSimilarityCalculator ::  GetDropTableString(string tableName)
//{
//	ostringstream oss;
//	
//    oss << "drop table " << tableName;
//
//	return oss.str();
//
//}
//
//string CDBSimilarityCalculator ::  GetInsertTableString(string tableName, double* data)
//{
//	ostringstream oss;
//	
//    oss << "insert into " << tableName << " (";
//
//	int i;
//	for (i = 0; i < numBins-1; i++)
//	{
//		oss << "t" << i << ",";
//	}
//    oss << "t" << i << ") values (";
//
//	for (i = 0; i < numBins-1; i++)
//	{
//		oss << data[i] << ",";
//	}
//    oss << data[i] << ")";
//
//	return oss.str();
//
//}

int CDBSimilarityCalculator ::  GetIndex(int i, int j)
{
    int index = i*numDims-(i-1)*i/2+j-i;
    return index;
}
