#include <cache/BrushEntries.h>
#include<cache/ICacheEntry.h>

using namespace cache;

bool iflessthan(ICacheEntry* a,ICacheEntry* b)
{
	if(a->x < b->x)
		return(true);
	else
		return(false);
}

BrushEntries::BrushEntries(std::list<ICacheEntry*>& listCacheEntry,
						   Brush *currentBrush):brush(*currentBrush)
{
	std::list<ICacheEntry*>::iterator it;
	dequeCacheEntryPtr  = new std::deque<ICacheEntry*>;

	//Insert all the entries into the listCacheEntry
	unsigned length = 0;
	for(it = listCacheEntry.begin();it != listCacheEntry.end(); it++)
	{
		this->dequeCacheEntryPtr->push_back(*it);
		length++;
	}
	
	std::sort(dequeCacheEntryPtr->begin(),
		dequeCacheEntryPtr->end(),
		iflessthan);

	std::deque<ICacheEntry*>::iterator checkit;
	ICacheEntry* cacheEntry; 
	for(checkit=dequeCacheEntryPtr->begin();checkit != dequeCacheEntryPtr->end();checkit++)
	{
		cacheEntry = *checkit;
	}

}


//Construts a new Brush entry
BrushEntries::BrushEntries(std::list<ICacheEntry*>& listCacheEntry,
						   Brush* currentBrush,
						   bool front):brush(*currentBrush)
{
	std::list<ICacheEntry*>::iterator it;

	//Insert all the entries into the listCacheEntry
	for(it = listCacheEntry.begin();it != listCacheEntry.end(); it++)
		this->dequeCacheEntryPtr->push_back(*it);
	
	std::sort(dequeCacheEntryPtr->begin(),
			  dequeCacheEntryPtr->end(),
			  iflessthan);

	this->front = front;
}

/*
 * Returns the left over brush after all the entries
 * have been removed
 */
Brush* BrushEntries::getLeftOverBrush()
{
	Brush* newbrush = NULL;
	double e1,e2;
	//Read the first element of the vector
	if(dequeCacheEntryPtr->size() > 0)
	{
		e1 = dequeCacheEntryPtr->front()->x;
		e2 = dequeCacheEntryPtr->back()->y;
		//Makes the new brush and sets  its values
		newbrush = new Brush(e1,e2,
							 brush.getLevelOfDetail(),
							 2,(int) brush.getBrushType());
		
	}
	//Form a new brush using those 2 elements.
	return(newbrush);
}

/*
* Returns the left end to the brush entry
* @return ICacheEntry*
*/
ICacheEntry* BrushEntries::getLeftEnd()
{
	return(this->dequeCacheEntryPtr->front());
}

/*
* Returns the right end to the brush entry
* @return ICacheEntry*
*/
ICacheEntry* BrushEntries::getRightEnd()
{
	return(this->dequeCacheEntryPtr->back());
}

/*
 * @ return usigned long
 * Returns the size of the list	
 */
unsigned long BrushEntries:: size()
{
	return(this->dequeCacheEntryPtr->size());
}
/*
 * Fetches the CacheEntry, Note the direction has to be initialized
 * before using this method.
 * @pre setDirection
 *	@return ICacheEntry*
 */
ICacheEntry* BrushEntries::getNext()
{
	ICacheEntry* result = NULL;
	
	if(front == true && it != itend)
	{
		result = *it;
		it++;
	}
	else if(front == false && it != itend)
	{
		result = *it;
		it--;
	}
	return(result);
}

/*
 *	Erases the brush entry 
 *  last fetched using getNext
 */
void BrushEntries::erase()
{
	ICacheEntry* cacheEntry;
	if(front)
	{
		//For debugging only
		cacheEntry = *(it-1);
		this->dequeCacheEntryPtr->erase(it-1);
	}
	else
	{
		//For debugging only
		cacheEntry = *(it+1);
		this->dequeCacheEntryPtr->erase(it+1);
	}
}

/**
 * Returns the brush for which the brush entries has a pointer
 * to the cache entries.
 * \return 
 */
Brush& BrushEntries::getBrush()
{
	return(this->brush);
}
/*
 *	Set leftToRight = true for lefttoright else set to false
 */
void BrushEntries::setDirection(bool leftToRight)
{
	
	this->front = leftToRight;
	if(front)
	{
		this->it = this->dequeCacheEntryPtr->begin();
		this->itend = this->dequeCacheEntryPtr->end();
	}	
	else
	{
		this->it = this->dequeCacheEntryPtr->end()-1;
		this->itend = this->dequeCacheEntryPtr->begin()-1;
	}
}

int* BrushEntries::getIDs(unsigned int& length)
{
	length = this->dequeCacheEntryPtr->size();
	int* ids = new int[length];
		
	std::deque<ICacheEntry*>::iterator checkit;
	int i = 0;
	for(checkit = this->dequeCacheEntryPtr->begin();
		checkit != dequeCacheEntryPtr->end();checkit++)
	{
		ids[i] = (*checkit)->getID();
		i++;
	}

	return(ids);
}


/*
 *	Destroys the brush entries
 */
BrushEntries ::~BrushEntries()
{
	delete(dequeCacheEntryPtr);
}

