#include <SpatialIndex.h>
#include <list>
#include <Tools.h>
#include <ICacheEntry.h>
#include <Brush.h>
#include <CacheSearcher.h>
#include <Mapper.h>
#include <ReplacementPolicy.h>
#include <Cache.h>

using namespace cache;

Cache:: Cache(long sizeOfCache,
			  ICacheEntry* buffer,
			  IIndex* index)
{		
	this->buffer = buffer;
	this->indexPtr = index;

	this->sizeOfCache =  sizeOfCache;			
	
	this->currentBrush = NULL;

	this->replacementPolicy = NULL;
}


void Cache:: addCacheEntry(ICacheEntry& cacheEntry)
{			
	this->indexPtr->insertData(cacheEntry);
	
	replacementPolicy->newData(cacheEntry);
}

/**
* Empties the cacheEntry, prepares it for containing a new object
* \param &cacheEntry 
*/
void Cache :: emptyCacheEntry(ICacheEntry &cacheEntry)	
{
	this->indexPtr->deleteData(cacheEntry.iShape,cacheEntry.id);
}

/*
* Searches for the brush in the cache.
*\param &brush
*\return std::list<ICacheEntry*>&
*/
std::list<ICacheEntry*>& Cache :: searchNode(Brush& brush)	
{
		
	std::list<ICacheEntry*>& resultList = this->indexPtr->search(brush);
	return(resultList);
}


/**
* Deletes the items from the cache
* \return 
*/
Cache ::~Cache()	
{	
	delete[] buffer;
}

/**
* This method is used by the ReplacementPolicy
* \param replacmentPolicy 
*/
void Cache :: setReplacementPolicy(ReplacementPolicy* replacementPolicy)
{
	this->replacementPolicy = replacementPolicy;
}


