#include <PrefetchController.h>
#include <Prefetcher.h>
#include <zthread/Thread.h>
#include <zthread/Mutex.h>
#include <IReplacementVisitor.h>
#include <iostream>
#include <PreEmptException.h>
#include <log/log.h>
#include <ICacheEntry.h>

using namespace cache;
/**
*This class is responsible for starting and stoping the prefetcher
*/

/**
* Constrcuts a new Prefetch controller
* \param *prefetcher 
* The prefetchController deletes the prefetcher.
* \return 
*/
PrefetchController::PrefetchController(Prefetcher *prefetcher)
:prefetchControllerLogger("PrefetchController","Logger for Prefetch Controller")
{
	this->prefetcher = prefetcher;
	this->prefetchThread = NULL;
	this->mutex = new ZThread::Mutex();				
	this->preEmptFlag = false;
	this->semPrefetchPermit = new ZThread::Semaphore(0,1);
	this->semLoadPermit = new ZThread::Semaphore(1,1);

	this->prefetcher->setSemaphore(this->semPrefetchPermit,
		this->semLoadPermit);

	//Instantiate the prefetch thread
	this->prefetchThread = new ZThread::Thread(this->prefetcher,true);							 		
}

/*
* This is used to get the load Permit just 
* used for experiment purposes. And used only in the testRTree.cpp.
* @Return ZThread::Semaphore
*/
ZThread::Semaphore* PrefetchController::getLoadPerimit()
{
	return(this->semLoadPermit);
}

//This will called in the context of the main thread
void PrefetchController:: preempt()
{	
	//This will not be NULL in 2 cases
	//a) The prefetcher exits by itself
	//b) The prefetcher is currently active
	// In case "a" wait will immediately return
	// In case "b" wait will return after chosenitem	
	mutex->acquire();		

//!!!!!!!!!!!!!!!!!!Have to uncomment if you want the prefetcher to preempt.!!!!!!!!!
//on a user request other wise the prefetcher and the user request will run in lock step.
	preEmptFlag = true;		
	mutex->release();			
	//Waiting for the prefetcher to give up CPU
	this->semLoadPermit->acquire();							

	#ifdef _DEBUG
		LOG_GMSG(prefetchControllerLogger,"Main thread has the CPU");			
	#endif //_DEBUG

	mutex->acquire();//Do not require this but still for safety.
	preEmptFlag = false;			
	mutex->release();							
}


//This is also called in the context of the main thread
void PrefetchController:: start()
{			
	this->semPrefetchPermit->release();
}

/**
*
* \param *cacheEntry 
* \return bool The return value decides whether the 
* prefetcher should continure or not
*/
bool PrefetchController:: chosenItem(ICacheEntry &cacheEntry)
{						
	bool flag = false;
	bool result = true;		
	this->mutex->acquire();
	//Check to see if got a signal from the main thread.
	if(this->preEmptFlag == true)
	{	
		flag = true;
	} 
	this->mutex->release();
	if(flag)
	{	
		//Prempt the prefetcher
		this->prefetcher->setPreEmpt(true);
	}
	return(prefetcher->ifContinue(cacheEntry));
}


PrefetchController:: ~PrefetchController()
{							
	this->prefetchThread->interrupt();
	//Waiting for thread to die
	this->prefetchThread->wait();
	delete(prefetchThread);
	delete(mutex);
	delete(this->semLoadPermit);
	delete(this->semPrefetchPermit);
}
