#include <cache/prefetcher/StatisticsGatherer.h>
#include <cache/prefetcher/Prefetcher.h>
#include <cache/prefetcher/PrefetchVisitor.h>
#include <strategyTable.h>
#include <ServedItemVisitor.h>
#include <ILoadListener.h>
#include <Brush.h>
#include <SpatialIndex.h>
#include <Timer/Timer.h>
#include <strategyTable.h>
#include <db_GLOBAL.h> //Defines all the macros for Prefetch Strategies
#include <sstream>
#include <ICacheEntry.h>
#include <log/log.h>
using namespace cache;

StatisticsGatherer::StatisticsGatherer():misPredicted(0),notPredicted(0),
correctlyPredicted(0),numberOfNodesPrefetched(0)
{
	statisticsLogger.enableMsgGroup();
	//Temporary************/
	//The assumption is that the statistics gatherer will only be used if
	//The strategy is adaptive prefetching
	// To initialize strategy table
	strategyTable_SelectPrefetchers();
}


/**
* Implements the served Item visitor.
* This method calculates the usage statistics
* Mainly the misspredicted count
* and the correctly predicted count.
* \param cacheEntry 
*/
void StatisticsGatherer::servedItem(ICacheEntry& cacheEntry) 
{		
	bool predicted = false;
	unsigned int size = (unsigned int)prefetchBrushes.size();
	Tools::Geometry::IShape* entryShape = cacheEntry.getIShape();		
	if(cacheEntry.ifNew())
	{
		this->notPredicted = this->notPredicted + 1;
		return;
	}
	for(unsigned int i=0;i<size;i++)
	{
		Tools::Geometry::IShape& brushShape = prefetchBrushes[i]->getShape();
		if(brushShape.intersectsShape(*entryShape))
		{
			correctlyPredicted = correctlyPredicted + 1;
			predicted = true;
			break;
		} // if(brushShape->intersectsShape(entryShape))	
	} // for(unsigned int i=0;i<size;i++)				

	//Note there can be still be items in the cache that are not predicted
	//The items that are not new but were already there in the cache and not 
	//predicted by any of the PRefetch brushes.
	if(!predicted)
	{
		this->notPredicted = notPredicted + 1;
	}
}

/**
* This will be called in the context of the main thread
* Implement the IEndLoadListener
* This is used to update the table with the usage statistics
*/
void StatisticsGatherer::stopLoad()
{
	//End the timer calculate the response time
	unsigned int respTime = timer.endTimer();
	if(correctlyPredicted > numberOfNodesPrefetched)		
		misPredicted = 0;	//Hack and have to remove this hack.
	//Add a isNew flag to the cacheEntry and reset that flag.
	else
		misPredicted = numberOfNodesPrefetched - correctlyPredicted;

	int selectedStrategy = strategyTable_GetSelectedStrategy();

	ostringstream strStream;

	strStream<<" Updating Statistics "
		<<" respTime "<<respTime
		<<" CorrectlyPredicted = "<<correctlyPredicted
		<<" Not Predicted = "<<notPredicted
		<<" MisPredicted = "<<misPredicted;			

	//Stores Logs the string
	#ifdef _DEBUG
	LOG_GMSG(this->statisticsLogger,strStream.str());
	#endif

	//This is directly copied from Punit's code		
	if ( ((numberOfNodesPrefetched != 0) && (selectedStrategy != NO_PREFETCH)) || (selectedStrategy == NO_PREFETCH) )
		strategyTable_UpdateTable3((float)respTime, correctlyPredicted, misPredicted, notPredicted) ;

	strategyTable_SelectStrategy() ;

	//And also you may require the current prefetching strategy.

}

/************************************************************************/
/*  This starts the  timer and used to keep track of the resp time      */ 
/*																		*/
/************************************************************************/
void StatisticsGatherer::startLoad()
{	
	timer.startTimer();
}

/**
* This will be called in the context of the prefetch thread
* Copy the state of the prefetcher so that it can be used 
* to calculate the statistics at the end of the next user load
* query
* \param Prefetcher
* \param *prefetcher 
*/
void StatisticsGatherer::prefetcherShuttingDown(Prefetcher *prefetcher)
{
	//Clearing the last state
	for(unsigned int i=0;i<prefetchBrushes.size();i++)
	{
		delete(prefetchBrushes[i]);
	}
	prefetchBrushes.clear();
	this->notPredicted = 0;
	this->correctlyPredicted = 0;
	this->misPredicted = 0;
	this->numberOfNodesPrefetched = 0;
	//Have to copy the brushes not very efficient but doing it 
	//at cost of good design, Hopefully we do not have many prefetch
	//brushes
	std::vector<Brush*>& vecPrefetchBrushes = 
		prefetcher->getPrefetchBrushes();

	for(unsigned int i=0;i<vecPrefetchBrushes.size();i++)
	{
		this->prefetchBrushes.
			push_back(new Brush(*vecPrefetchBrushes[i]));
		this->numberOfNodesPrefetched = prefetcher->getNumberOfNodesPrefetched();
	}
}
/**
* Destroys the statistics gatherer
* \return 
*/
StatisticsGatherer::~StatisticsGatherer(){	
	for(unsigned int i=0;i<prefetchBrushes.size();i++)
	{
		delete(prefetchBrushes[i]);
	}
	prefetchBrushes.clear();
}
